from pathlib import Path
from .datatypes import (
    CStruct,
    CEnum,
    CEnumVal,
    CFunc,
    CHeader,
    CSource,
    CArg,
    CType,
    CPointer,
    CPointerType,
    CQualifier,
    CInclude,
    CUserType,
)


def test_str8():
    struct = CStruct(
        name="Str8",
        cargs=[
            CArg(name="size", _type=CType.U64),
            CArg(name="capacity", _type=CType.U64),
            CArg(name="buf", _type=CType.U8, pointer=CPointer(_type=CPointerType.SINGLE)),
        ],
    )

    cenum = CEnum(
        name="OS",
        values=[
            CEnumVal(name="OS_LINUX"),
            CEnumVal(name="OS_WINDOWS"),
            CEnumVal(name="OS_MACOS"),
        ]
    )

    typed_enum = CEnum(
        name="Compiler",
        values=[
            CEnumVal(name="COMPILER_GCC"),
            CEnumVal(name="COMPILER_CLANG"),
            CEnumVal(name="COMPILER_MSVC"),
        ],
        typedef=True,
    )

    main_func = CFunc(
        name="my_custom_func",
        ret_type=CType.I32,
        args=[
            CArg(name="argc", _type=CType.I32),
            CArg(name="argv", _type=CType.CHAR, pointer=CPointer(_type=CPointerType.DOUBLE), qualifier=CQualifier.CONST),
        ],
        body="  return 0;"
    )

    header = CHeader(
        name="str",
        includes=[
            CInclude(header="aliases.h", local=True),
        ],
        types=[struct, cenum, typed_enum],
        funcs=[main_func],
    )

    source = CSource(
        name="str",
        includes=[
            CInclude(header=header, local=True),
            CInclude(header="aliases.h", local=True),
        ],
        types=[],
        internal_funcs=[],
        funcs=[main_func]
    )

    header.save(Path("."))
    source.save(Path("."))


def test_typed_array():
    datatypes = [CType.U8, CType.F32, CType.CHAR]
    types: list[CUserType] = []
    funcs: list[CFunc] = []

    getter_func_body = """\
  if(idx >= arr->size) {
    return 0;
  }

  return arr->data[idx];"""

    setter_func_body = """\
  if(idx >= arr->size) {
    return;
  }

  arr->data[idx] = val;"""

    for _type in datatypes:
        struct = CStruct(
            name=f"Array{str(_type).title()}".strip(),
            cargs=[
                CArg(name="size", _type=CType.U64),
                CArg(name="data", _type=_type, pointer=CPointer(_type=CPointerType.SINGLE)),
            ],
        )
        types.append(struct)

        funcs.append(CFunc(
            name=f"array_{str(_type)}_get",
            ret_type=_type,
            args=[
                CArg(name="arr", _type=struct, pointer=CPointer(_type=CPointerType.SINGLE), qualifier=CQualifier.CONST),
                CArg(name="idx", _type=CType.U64),
            ],
            body=getter_func_body,
        ))

        funcs.append(CFunc(
            name=f"array_{str(_type)}_set",
            ret_type=CType.VOID,
            args=[
                CArg(name="arr", _type=struct, pointer=CPointer(_type=CPointerType.SINGLE)),
                CArg(name="idx", _type=CType.U64),
                CArg(name="val", _type=_type)
            ],
            body=setter_func_body,
        ))

    header = CHeader(
        name="typed_array",
        includes=[
            CInclude(header="aliases.h", local=True)
        ],
        types=types,
        funcs=funcs
    )

    source = CSource(
        name="typed_array",
        includes=[
            CInclude(header="aliases.h", local=True),
            CInclude(header=header, local=True)
        ],
        funcs=funcs
    )

    header.save(Path("."))
    source.save(Path("."))
